@@ -13,7 +13,7 @@
 
 package require Tcl 8.2
 package require cmdline
-package provide fileutil 1.14.8
+package provide fileutil 1.15
 
 namespace eval ::fileutil {
     namespace export \
@@ -21,7 +21,7 @@ namespace eval ::fileutil {
 	    jail stripPwd stripN stripPath tempdir tempfile \
 	    install fileType writeFile appendToFile \
 	    insertIntoFile removeFromFile replaceInFile \
-	    updateInPlace test tempdirReset
+	    updateInPlace test tempdirReset maketempdir
 }
 
 # ::fileutil::grep --
@@ -89,7 +89,6 @@ proc ::fileutil::find {{basedir .} {filtercmd {}}} {
 	FADD $basedir
 
     } elseif {[file isdirectory $basedir]} {
-
 	# For a directory as base we do an iterative recursion through
 	# the directory hierarchy starting at the base. We use a queue
 	# (Tcl list) of directories we have to check. We access it by
@@ -113,7 +112,9 @@ proc ::fileutil::find {{basedir .} {filtercmd {}}} {
 
 	set pending [list $basedir]
 	set at      0
-	array set   known {}
+	array set   parent {}
+	array set   norm   {}
+	Enter {} $basedir
 
 	while {$at < [llength $pending]} {
 	    # Get next directory not yet processed.
@@ -144,9 +145,8 @@ proc ::fileutil::find {{basedir .} {filtercmd {}}} {
 		# encountered before. If ok, record directory for
 		# expansion in future iterations.
 
-		set norm [fileutil::fullnormalize $f]
-		if {[info exists known($norm)]} continue
-		set known($norm) .
+		Enter $current $f
+		if {[Cycle $f]} continue
 
 		lappend pending $f
 	    }
@@ -158,6 +158,24 @@ proc ::fileutil::find {{basedir .} {filtercmd {}}} {
     return $result
 }
 
+proc  ::fileutil::Enter {parent path} {
+    upvar 1 parent _parent norm _norm
+    set _parent($path) $parent
+    set _norm($path)   [fullnormalize $path]
+    return
+}
+
+proc  ::fileutil::Cycle {path} {
+    upvar 1 parent _parent norm _norm
+    set nform $_norm($path)
+    set paren $_parent($path)
+    while {$paren ne {}} {
+	if {$_norm($paren) eq $nform} { return yes }
+	set paren $_parent($paren)
+    }
+    return no
+}
+
 # Helper command for fileutil::find. Performs the filtering of the
 # result per a filter command for the candidates found by the
 # traversal core, see above. This is portable.
@@ -220,12 +238,8 @@ if {[package vsatisfies [package present Tcl] 8.5]} {
     proc ::fileutil::ACCESS {args} {}
 
     proc ::fileutil::GLOBF {current} {
-	if {![file readable $current]} {
-	    return {}
-	}
-	if {([file type $current] eq "link") &&
-	    !([file exists   [file readlink $current]] &&
-	      [file readable [file readlink $current]])} {
+	if {![file readable $current] ||
+	    [BadLink $current]} {
 	    return {}
 	}
 
@@ -245,20 +259,28 @@ if {[package vsatisfies [package present Tcl] 8.5]} {
     }
 
     proc ::fileutil::GLOBD {current} {
-	if {![file readable $current]} {
-	    return {}
-	}
-	if {([file type $current] eq "link") &&
-	    !([file exists   [file readlink $current]] &&
-	      [file readable [file readlink $current]])} {
+	if {![file readable $current] ||
+	    [BadLink $current]} {
 	    return {}
 	}
 
 	lsort -unique [concat \
-	    [glob -nocomplain -directory $current -types d          -- *] \
-	    [glob -nocomplain -directory $current -types {hidden d} -- *]]
+	   [glob -nocomplain -directory $current -types d          -- *] \
+	   [glob -nocomplain -directory $current -types {hidden d} -- *]]
     }
 
+    proc ::fileutil::BadLink {current} {
+	if {[file type $current] ne "link"} { return no }
+
+	set dst [file join [file dirname $current] [file readlink $current]]
+
+	if {![file exists   $dst] ||
+	    ![file readable $dst]} {
+	    return yes
+	}
+
+	return no
+    }
 } elseif {[package vsatisfies [package present Tcl] 8.4]} {
     # Tcl 8.4+.
     # (Ad 1) We have -directory, and -types,
@@ -1398,25 +1420,24 @@ proc ::fileutil::foreachLine {var filename cmd} {
     catch {
 	set code 0
 	set result {}
+	set return 0
 	while {[gets $fp line] >= 0} {
-	    set code [catch {uplevel 1 $cmd} result]
-	    if {($code != 0) && ($code != 4)} {break}
+	    set code [catch {uplevel 1 $cmd} result options]
+	    if {$code == 2} {
+		set return 1
+		set code [dict get $options -code]
+		break
+	    } elseif {$code != 0 && $code != 4} {
+		break
+	    }
 	}
     }
     close $fp
 
-    if {($code == 0) || ($code == 3) || ($code == 4)} {
-        return $result
+    if {$return || $code == 1 || $code > 4} {
+	return -options $options $result
     }
-    if {$code == 1} {
-        global errorCode errorInfo
-        return \
-		-code      $code      \
-		-errorcode $errorCode \
-		-errorinfo $errorInfo \
-		$result
-    }
-    return -code $code $result
+    return $result
 }
 
 # ::fileutil::touch --
@@ -1634,10 +1655,10 @@ proc ::fileutil::fileType {filename} {
         lappend type graphic tiff
     } elseif { $binary && [string match "BM*" $test] && [string range $test 6 9] == "\x00\x00\x00\x00" } {
         lappend type graphic bitmap
-    } elseif { $binary && [string match "\%PDF\-*" $test] } {
-        lappend type pdf
     } elseif { ! $binary && [string match -nocase "*\<html\>*" $test] } {
         lappend type html
+    } elseif {[string match "\%PDF\-*" $test] } {
+        lappend type pdf
     } elseif { [string match "\%\!PS\-*" $test] } {
        lappend type ps
        if { [string match "* EPSF\-*" $test] } {
@@ -1842,6 +1863,56 @@ namespace eval ::fileutil {
     variable tempdirSet 0
 }
 
+# ::fileutil::maketempdir --
+
+proc ::fileutil::maketempdir {args} {
+    return [Normalize [MakeTempDir $args]]
+}
+
+proc ::fileutil::MakeTempDir {config} {
+    # Setup of default configuration.
+    array set options {}
+    set options(-suffix) ""
+    set options(-prefix) "tmp"
+    set options(-dir)    [tempdir]
+
+    # TODO: Check for and reject options not in -suffix, -prefix, -dir
+    # Merge user configuration, overwrite defaults.
+    array set options $config
+
+    # See also "tempfile" below. Could be shareable internal configuration.
+    set chars       abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789
+    set nrand_chars 10
+    set maxtries    10
+
+    for {set i 0} {$i < $maxtries} {incr i} {
+	# Build up the candidate name. See also "tempfile".
+	set directory_name $options(-prefix)
+	for {set j 0} {$j < $nrand_chars} {incr j} {
+	    append directory_name \
+		[string index $chars [expr {int(rand() * 62)}]]
+	}
+	append directory_name $options(-suffix)
+	set path [file join $options(-dir) $directory_name]
+
+	# Try to create. Try again if already exists, or trouble
+	# with creation and setting of perms.
+	#
+	# Note: The last looks as if it is able to leave partial
+	# directories behind (created, trouble with perms). But
+	# deleting ... Might pull the rug out from somebody else.
+
+	if {[file exists $path]} continue
+	if {[catch {
+	    file mkdir $path
+	    file attributes $path -permissions 0700
+	}]} continue
+
+	return $path
+    }
+    return -code error "Failed to find an unused temporary directory name"
+}
+
 # ::fileutil::tempfile --
 #
 #   generate a temporary file name suitable for writing to
